@@ -33,7 +33,8 @@
 #include "gal/util/e-util.h"
 #include "e-table-item.h"
 #include "e-cell-popup.h"
-
+#include "gal/a11y/e-table/gal-a11y-e-cell-popup.h"
+#include "gal/a11y/e-table/gal-a11y-e-cell-registry.h"
 
 #define E_CELL_POPUP_ARROW_WIDTH	16
 #define E_CELL_POPUP_ARROW_XPAD		3
@@ -139,7 +140,10 @@ e_cell_popup_class_init		(GtkObjectClass	*object_class)
 	ecc->show_tooltip = ecp_show_tooltip;
 	ecc->get_bg_color = ecp_get_bg_color;
 
-	parent_class = gtk_type_class (e_cell_get_type ());
+	parent_class = g_type_class_ref (E_CELL_TYPE);
+	gal_a11y_e_cell_registry_add_cell_type (NULL,
+                                                E_CELL_POPUP_TYPE,
+                                                gal_a11y_e_cell_popup_new);
 }
 
 
@@ -161,7 +165,7 @@ e_cell_popup_init		(ECellPopup	*ecp)
 ECell *
 e_cell_popup_new		(void)
 {
-	ECellPopup *ecp = gtk_type_new (e_cell_popup_get_type ());
+	ECellPopup *ecp = g_object_new (E_CELL_POPUP_TYPE, NULL);
 
 	return (ECell*) ecp;
 }
@@ -176,7 +180,7 @@ e_cell_popup_dispose (GObject *object)
 	ECellPopup *ecp = E_CELL_POPUP (object);
 
 	if (ecp->child)
-		gtk_object_unref (GTK_OBJECT (ecp->child));
+		g_object_unref (ecp->child);
 	ecp->child = NULL;
 
 	G_OBJECT_CLASS (parent_class)->dispose (object);
@@ -266,22 +270,18 @@ ecp_draw (ECellView *ecv, GdkDrawable *drawable,
 	GtkWidget *canvas = GTK_WIDGET (GNOME_CANVAS_ITEM (ecv->e_table_item_view)->canvas);
 	GtkShadowType shadow;
 	GdkRectangle rect;
-	gboolean show_popup_arrow = FALSE;
+	gboolean show_popup_arrow;
 
 	/* Display the popup arrow if we are the cursor cell, or the popup
 	   is shown for this cell. */
-	if (flags & E_CELL_CURSOR) {
-		show_popup_arrow = TRUE;
-		ecp->popup_arrow_shown = TRUE;
-	} else if (ecp->popup_shown && ecp->popup_view_col == view_col
-		   && ecp->popup_row == row && ecp->popup_model == ((ECellView *) ecp_view)->e_table_model) {
-		show_popup_arrow = TRUE;
-	}
+	show_popup_arrow = e_table_model_is_cell_editable (ecv->e_table_model, model_col, row) &&
+		(flags & E_CELL_CURSOR ||
+		 (ecp->popup_shown && ecp->popup_view_col == view_col
+		  && ecp->popup_row == row
+		  && ecp->popup_model == ((ECellView *) ecp_view)->e_table_model));
 
-#if 0
-	g_print ("In ecp_draw row:%i col: %i %i,%i %i,%i Show Arrow:%i\n",
-		 row, view_col, x1, y1, x2, y2, show_popup_arrow);
-#endif
+	if (flags & E_CELL_CURSOR)
+		ecp->popup_arrow_shown = show_popup_arrow;
 
 	if (show_popup_arrow) {
 		e_cell_draw (ecp_view->child_view, drawable, model_col,
@@ -303,7 +303,7 @@ ecp_draw (ECellView *ecv, GdkDrawable *drawable,
 			       &rect, canvas, "ecellpopup",
 			       rect.x, rect.y, rect.width, rect.height);
 		gtk_paint_arrow (canvas->style, drawable,
-				 GTK_STATE_NORMAL, shadow,
+				 GTK_STATE_NORMAL, GTK_SHADOW_NONE,
 				 &rect, canvas, NULL,
 				 GTK_ARROW_DOWN, TRUE,
 				 rect.x + E_CELL_POPUP_ARROW_XPAD,
@@ -331,14 +331,12 @@ ecp_event (ECellView *ecv, GdkEvent *event, int model_col, int view_col,
 
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
-		if (flags & E_CELL_CURSOR
+		if (e_table_model_is_cell_editable (ecv->e_table_model, model_col, row) &&
+		    flags & E_CELL_CURSOR
 		    && ecp->popup_arrow_shown) {
 			width = e_table_header_col_diff (eti->header, view_col,
 							 view_col + 1);
 
-			g_print ("Event in item popup width: %i button: %g,%g\n",
-				 width, event->button.x, event->button.y);
-
 			/* FIXME: The event coords seem to be relative to the
 			   text within the cell, so we have to add 4. */
 			if (event->button.x + 4 >= width - E_CELL_POPUP_ARROW_WIDTH) {
@@ -346,15 +344,12 @@ ecp_event (ECellView *ecv, GdkEvent *event, int model_col, int view_col,
 			}
 		}
 		break;
-	case GDK_BUTTON_RELEASE:
-		break;
 	case GDK_KEY_PRESS:
-		if (event->key.state & GDK_MOD1_MASK
+		if (e_table_model_is_cell_editable (ecv->e_table_model, model_col, row) &&
+		    event->key.state & GDK_MOD1_MASK
 		    && event->key.keyval == GDK_Down) {
-			g_print ("## Alt-Down pressed\n");
 			return e_cell_popup_do_popup (ecp_view, event, row, view_col);
 		}
-		g_print ("Key Press Event ECellPopup\n");
 		break;
 	default:
 		break;
@@ -385,9 +380,6 @@ ecp_enter_edit (ECellView *ecv, int model_col, int view_col, int row)
 {
 	ECellPopupView *ecp_view = (ECellPopupView *) ecv;
 
-	g_print ("In ecp_enter_edit model_col: %i view_col: %i row: %i\n",
-		 model_col, view_col, row);
-
 	return e_cell_enter_edit (ecp_view->child_view, model_col, view_col, row);
 }
 
@@ -401,9 +393,6 @@ ecp_leave_edit (ECellView *ecv, int model_col, int view_col, int row,
 {
 	ECellPopupView *ecp_view = (ECellPopupView *) ecv;
 
-	g_print ("In ecp_leave_edit model_col: %i view_col: %i row: %i\n",
-		 model_col, view_col, row);
-
 	e_cell_leave_edit (ecp_view->child_view, model_col, view_col, row,
 			   edit_context);
 }
@@ -483,10 +472,10 @@ e_cell_popup_set_child			(ECellPopup	*ecp,
 	g_return_if_fail (E_IS_CELL_POPUP (ecp));
 
 	if (ecp->child)
-		gtk_object_unref (GTK_OBJECT (ecp->child));
+		g_object_unref (ecp->child);
 
 	ecp->child = child;
-	gtk_object_ref (GTK_OBJECT (child));
+	g_object_ref (child);
 }
 
 
